/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIPrincipal;
interface nsIURI;

/**
 * nsIURIWithPrincipal is implemented by URIs which are associated with a
 * specific principal.
 */
[scriptable, uuid(626a5c0c-bfd8-4531-8b47-a8451b0daa33)]
interface nsIURIWithPrincipal : nsISupports
{
    /**
     * The principal associated with the resource returned when loading this
     * uri.
     */
    readonly attribute nsIPrincipal principal;

    /**
     * The uri for the principal.
     */
    readonly attribute nsIURI principalUri;
};
